@@ -1,4 +1,4 @@
-__version__ = "2.1.2"
+__version__ = "2.1.4"
 __author__ = "Jiace Sun"
 
 import os
@@ -6,7 +6,7 @@ ROOT = os.path.dirname(os.path.abspath(__file__))
 from . import config
 from . import translate
 from . import tencent
-from . import fix_encoding
+from . import encoding
 from . import process_latex
 from . import process_text
 from . import translate_tex
